<?php


namespace Drupal\tmgmt_xtm\Plugin\tmgmt\Translator;


use Masterminds\HTML5\Exception;

/**
 * Class XmlFilesProvider
 * @package Drupal\tmgmt_xtm\Plugin\tmgmt\Translator
 *
 * Provide XML files from obtained XTM file
 */
class XmlFilesProvider
{
  /**
   * @var
   */
  private $xtmFile;

  /**
   * XmlFilesProvider constructor.
   * @param $xtmFile
   */
  public function __construct($xtmFile)
  {
    $this->xtmFile = $xtmFile;
  }

  /**
   * @return array
   * @throws Exception
   */
  public function getXmlFiles()
  {
    $this->unpack();
    $xmls = $this->loadXmls();
    $this->removeFolder();
    if (empty($xmls)) {
      throw new Exception("Could not read XTM file from Zip");
    }
    return $xmls;
  }

  /**
   *
   */
  private function unpack()
  {
    ZipHelper::unpack(FileHelper::get_temp($this->xtmFile), ZipHelper::get_unpack_folder($this->xtmFile));
  }

  /**
   * @return array|false
   */
  private function findPaths()
  {
    $globFilePaths = glob(ZipHelper::get_unpack_folder($this->xtmFile) . '/' . $this->xtmFile->targetLanguage . '/' . "*.xml");
    if (empty($globFilePaths)) {
      $globFilePaths = glob(ZipHelper::get_unpack_folder($this->xtmFile) . '/' . "*.xml");
    }
    return $globFilePaths;
  }

  /**
   * @return array
   * @throws Exception
   */
  private function loadXmls()
  {
    $xmls = [];
    foreach ($this->findPaths() as $globFilePath) {
      $xml = simplexml_load_string(file_get_contents($globFilePath), null, LIBXML_NOCDATA);
      if (false === $xml) {
        throw new Exception("No XML DATA");
      }
      $xmls[] = $xml;
    }
    return $xmls;
  }

  /**
   *
   */
  private function removeFolder()
  {
    $filename = ZipHelper::get_unpack_folder($this->xtmFile);
    if (is_dir($filename)) {
      FileHelper::remove_folder($filename);
    }
  }
}
